/*
 *   Copyright © 2011 ROSA Lab. <ural.mullabaev@rosalab.ru>
 *   Copyright © 2008, 2009 Fredrik Höglund <fredrik@kde.org>
 *   Copyright © 2008 Rafael Fernández López <ereslibre@kde.org>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public License
 *   along with this library; see the file COPYING.LIB.  If not, write to
 *   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *   Boston, MA 02110-1301, USA.
 */

#ifndef STACKFOLDER_H
#define STACKFOLDER_H

#include <Solid/Networking>

#include <plasma/applet.h>

#include "iconview.h"

namespace Plasma
{
    class IconWidget;
}

class KFilePlacesModel;
class KFilePreviewGenerator;
class QGraphicsLinearLayout;
class ProxyModel;
class Dialog;
class IconView;
class Label;


class StackFolder : public Plasma::Applet
{
    Q_OBJECT

public:
    StackFolder(QObject *parent, const QVariantList &args);
    ~StackFolder();

    void init();
    void saveState(KConfigGroup &config) const;

    void configChanged();

protected:
    void constraintsEvent(Plasma::Constraints);
    QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint) const;
    bool eventFilter(QObject *watched, QEvent *event);
    void timerEvent(QTimerEvent *event);

protected slots:
    void refreshIcons();

    void updateIconWidget();
    void iconWidgetClicked();
    void returnIconClicked();
    void openIconClicked();
    void activated(const QModelIndex &index);
    void indexesMoved(const QModelIndexList &indexes);
    void dialogHidden();
    void folderChanged(const KUrl& url);
    void fontSettingsChanged();
    void iconSettingsChanged(int group);
    void iconGeometryChanged();
    void themeChanged();
    void networkStatusChanged(Solid::Networking::Status status);

private:
    QColor textColor() const;
    void setUrl(const KUrl &url);
    void changeFolder(const KUrl &url, const bool toChild);
    QSize iconSize() const;
    void updateIconViewState();
    void saveIconPositions() const;
    void showDialog();
    void hideDialog();

private:
    Dialog *m_dialog;
    QGraphicsWidget *m_graphicsWidget;
    KFileItemDelegate *m_delegate;
    QPointer<KFilePreviewGenerator> m_previewGenerator;
    QItemSelectionModel *m_selectionModel;
    ProxyModel *m_model;
    KDirModel *m_dirModel;
    KFilePlacesModel *m_placesModel;
    Plasma::IconWidget *m_iconWidget;
    Plasma::IconWidget *m_returnIcon;
    Plasma::IconWidget *m_openIcon;
    Label *m_label;
    IconView *m_iconView;
    QGraphicsLinearLayout *m_layout;
    KUrl m_url;
    KUrl m_topUrl;
    QColor m_textColor;
    QString m_titleText;
    QBasicTimer m_delayedShowTimer;
    QBasicTimer m_delayedSaveTimer;
    int m_filterType;
    int m_sortColumn;
    bool m_sortDirsFirst;
    bool m_showPreviews;
    bool m_drawShadows;
    bool m_iconsLocked;
    bool m_alignToGrid;
    bool m_hideForChangeFolder;
    QStringList m_previewPlugins;
    int m_customIconSize;
    int m_numTextLines;
    IconView::Flow m_flow;
    bool m_needShow;
};

#endif
